import os
import json
import yaml


def generate_directory_structure(start_path, output_format="json"):
    """
    Generate a directory structure for the specified directory.

    Args:
        start_path (str): The directory to inspect.
        output_format (str): The desired output format ('json', 'yaml', or 'txt').

    Returns:
        str: The directory structure in the specified format.

    Raises:
        FileNotFoundError: If the specified directory does not exist.
    """
    try:
        if not os.path.exists(start_path):
            raise FileNotFoundError(f"The specified directory '{start_path}' does not exist.")

        structure = {}
        to_ignore = _read_gitignore(start_path)

        for dirpath, dirnames, filenames in os.walk(start_path):
            for r in to_ignore:
                if r in dirnames:
                    dirnames.remove(r)

            current_dir = structure
            path_components = os.path.relpath(dirpath, start_path).split(os.path.sep)

            for component in path_components:
                if component not in current_dir:
                    current_dir[component] = {}
                current_dir = current_dir[component]

            for filename in filenames:
                current_dir[filename] = None

        if output_format == "json":
            return json.dumps(structure, indent=4)
        elif output_format == "yaml":
            return yaml.dump(structure, default_flow_style=False)
        elif output_format == "txt":
            return _generate_txt_structure(structure, "")

    except Exception as e:
        return f"Error: {str(e)}"


def save_structure_to_file(output_file, structure):
    """
    Save the directory structure to a file.

    Args:
        output_file (str): The name of the output file.
        structure (str): The directory structure content to be saved.

    Returns:
        str: An error message if an error occurs during file saving; otherwise, None.
    """
    try:
        with open(output_file, 'w') as file:
            file.write(structure)
        return None  # No error occurred
    except Exception as e:
        return f"Error: {str(e)}"


def _read_gitignore(start_path):
    """
    Read patterns from a .gitignore file in the specified directory.

    Args:
        start_path (str): The directory to read .gitignore from.

    Returns:
        list: List of patterns to be ignored.
    """
    gitignore_path = os.path.join(start_path, ".gitignore")
    to_ignore = [
        ".env",
        "__pycache__",
        ".pytest_cache",
        ".vscode",
        ".git",
    ]

    if os.path.exists(gitignore_path):
        with open(gitignore_path, "r") as gitignore_file:
            lines = gitignore_file.readlines()
            for line in lines:
                line = line.strip()
                if line and not line.startswith("#"):
                    to_ignore.append(line)

    return to_ignore


def _generate_txt_structure(structure, indentation):
    """
    Generate a plain text representation of the directory structure.

    Args:
        structure (dict): The directory structure.
        indentation (str): The current indentation level.

    Returns:
        str: The plain text representation of the directory structure.
    """
    result = ""
    for key, value in structure.items():
        if value is None:
            result += indentation + f"- {key}\n"
        else:
            result += indentation + f"- {key}:\n"
            result += _generate_txt_structure(value, indentation + "  ")
    return result



from setuptools import setup

# Leer la descripción larga desde README.md
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    long_description=long_description,
)



import argparse
import os
import sys
from _directory_structure import generate_directory_structure as get_project_structure
from _directory_structure import save_structure_to_file
from _pretty_structure import pretty_print


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Generate a directory structure in different formats.")
    parser.add_argument("directory", nargs="?", default=os.getcwd(), help="The directory to inspect.")
    parser.add_argument("-o", "--output", help="The output file name.")
    parser.add_argument("-f", "--format", choices=["json", "yaml", "txt"], default="yaml", help="The output file format.")
    parser.add_argument("-p", "--print", action="store_true", help="Print the directory structure without saving it.")
    parser.add_argument("-ns", "--not-save", action="store_true", help="Structure won't be saved in output file.")

    args = parser.parse_args()

    start_path = args.directory
    output_file = args.output
    output_format = args.format

    if output_file is None:
        # Use the directory name as the default output file name
        directory_name = os.path.basename(os.path.normpath(start_path))
        output_file = directory_name

    # Convert relative path to absolute path if necessary
    start_path = os.path.abspath(start_path)

    if output_format == "json":
        output_file += '.json'
    elif output_format == "yaml":
        output_file += '.yaml'
    elif output_format == "txt":
        output_file += '.txt'

    structure = get_project_structure(start_path, output_format)

    if structure is not None:

        if args.print:
            print_strcuture = get_project_structure(start_path, "json")
            pretty_print(print_strcuture)

        if "Error" in structure:
            print(structure)  # Print the error message
            sys.exit(1)
        else:
            if not args.not_save:
                error = save_structure_to_file(output_file, structure)
                if error:
                    print(error)  # Print the error message
                else:
                    print(f"Directory structure has been generated and saved to '{output_file}'.")
